/*
 *      L I S T _ A d d
 *
 *  This function is called when it is required
 *  to insert a new node in order in any list.
 *
 *  If it is called on an unordered list we
 *  simply revert to adding at the end.
 *
 *
 * (c) Copyright 1996 David J. Walker
 *     Email:  d.j.walker@slh0101.wins.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  22 May 96   DJW   - First version
 *
 *  31 May 96   DJW   - Amended for data structure changes to support
 *                      the 'Clone' method.
 */

#include <liblist.h>
#include <sys/liblist.h>
#include <errno.h>


int     LIST_Add (list_t list, node_t node)
{
    register  _listclass_t    class;
    register  node_t lastnode;

    debug ("LIST_Add", \
           "'list' parameter is not a valid list instance", \
           _LIST_ListType(list));
    debug ("LIST_Add", \
           "'node' parameter is not a valid member of this list type", \
           _LIST_ListId(list,node));
    debug ("LIST_Add", \
           "node is already a member of this list", \
           LIST_Index(list,node) == 0);

    class = list->class;

    node = _LIST_REALNODE(list,node);

    if (class->compare != NULL)
    {
        /*
         *  If this is an ordered list, then
         *  use the list specific add routine.
         */
        return (*(class->master->add))(list, node);
    }

    /*
     *  If this is an unordered list, then
     *  we always insert at the end of the list.
     */
    if ((lastnode = LIST_Last(list)) != NULL)
    {
        lastnode = _LIST_REALNODE(list, lastnode);
    }
    return  (*(class->master->after))(list, node, lastnode);
}

