/*
 *      L I S T _ A f t e r
 *
 *  This function is called when it is required
 *  to insert a new node after a specifed node.
 *
 *
 * (c) Copyright 1996 David J. Walker
 *     Email:  d.j.walker@slh0101.wins.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  22 May 96   DJW   - First version
 */

#include <liblist.h>
#include <sys/liblist.h>
#include <errno.h>

int     LIST_After (list_t list, node_t newnode, node_t oldnode)
{
    _listclass_t class;

    debug ("LIST_After", \
            "'list' parameter is not a valid list pointer", \
            _LIST_ListType(list));
    debug ("LIST_After", \
            "'newnode' parameter is not a valid member of this list type",\
            _LIST_ListId(list,newnode));
    debug ("LIST_After", \
            "'newnode is already a member of this list", \
             LIST_Index(list,newnode) == 0);

    class = list->class;


#ifdef DEBUG
    if (oldnode != NULL)
    {
        debug ("LIST_After", \
                "'oldnode is not a member of this list", \
                 LIST_Index(list,oldnode) == 0);
    }
#endif /* DEBUG */

    /*
     *  There is a special case of
     *  'oldnode' being NULL which
     *  means insert at end of list.
     */
    if (oldnode == NULL)
    {
        oldnode = LIST_Last(list);
    }

    /*
     *  First see if this is an ordered list
     *  as then we need to do some checks on
     *  whether it is valid to insert at
     *  this point.
     */
    if (class->compare)
    {
        if (oldnode != NULL)
        {
            node_t   tempnode;

            if (LIST_Compare(list,newnode,oldnode) < 0)
            {
                return (EINVAL);
            }
            tempnode = LIST_Next(list, oldnode);
            if (tempnode != NULL)
            {
                if (LIST_Compare(list,newnode,tempnode) > 0)
                {
                    return (EINVAL);
                }
            }
        }
    }
    /*
     *  either we passed checks or list is unordered.
     *  Call type specific support routine to add node.
     */
    newnode = _LIST_REALNODE(list,newnode);
    if (oldnode != NULL)
    {
        oldnode = _LIST_REALNODE(list,oldnode);
    }
    return (class->master->after(list,newnode,oldnode));
}

