/*
 *      L I S T _ B e f o r e
 *
 *  This function is called when it is required
 *  to insert a new node before a specifed node.
 *
 *  We have actually only really implemented inserting
 *  nodes after a given node in the support routines,
 *  so we do the work to effectively change this into
 *  a List_After() type routine.
 *
 *
 * (c) Copyright 1996 David J. Walker
 *     Email:  d.j.walker@slh0101.wins.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  22 May 96   DJW   - First version
 */

#define __LIBLIST__
#include <liblist.h>
#include <sys/liblist.h>
#include <errno.h>

int     LIST_Before (list_t list, node_t newnode, node_t oldnode)
{
    node_t   tempnode;
    _listclass_t class;

    debug ("LIST_Before", \
            "'list' parameter is not a valid list pointer", \
            _LIST_ListType(list));
    debug ("LIST_Before", \
            "'newnode' parameter is not a valid member of this list type",\
            _LIST_ListId(list,newnode));
    debug ("LIST_Before", \
            "'newnode is already a member of this list", \
             LIST_Index(list,newnode) == 0);

    class = list->class;
    /*
     *  There is a special case of
     *  'oldnode' being NULL which
     *  means insert at front of list.
     */
    if (oldnode == NULL)
    {
        oldnode = LIST_First(list);
    }

    if (oldnode == NULL)
    {
        /*
         *  Looks like an empty list!
         */

        tempnode = NULL;
    }
    else
    {
        debug ("LIST_Before", \
                "'oldnode' is already a member of this list", \
                 LIST_Index(list,newnode) == 0);


        tempnode = LIST_Previous(list, oldnode);

        /*
         *  First see if this is an ordered list
         *  as then we need to do some checks on
         *  whether it is valid to insert at
         *  this point.
         */
        if (class->compare)
        {
            if (oldnode != NULL)
            {

                if (LIST_Compare(list,newnode,oldnode) > 0)
                {
                    return (EINVAL);
                }
                if (tempnode != NULL)
                {
                    if (LIST_Compare(list,newnode,tempnode) < 0)
                    {
                        return (EINVAL);
                    }
                }
            }
        }
        tempnode = _LIST_REALNODE(list,tempnode);
    }
    newnode = _LIST_REALNODE(list,newnode);
    /*
     *  either we passed checks or list is unordered.
     *  Call type specific support routine to add node.
     */
    return ((*(class->master->after))(list,newnode,tempnode));
}

