/*
 *      b t r e e  _ c
 *
 *  This contains routines that are specific
 *  to the implementation of binary trees
 *
 *
 * (c) Copyright 1996 David J. Walker
 *     Email:  d.j.walker@slh0101.wins.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  12 May 96   DJW   - First version
 */

#include <liblist.h>
#include <sys/liblist.h>


/*======================================================= BTREE_ADDNODE */
static
void    BTREE_AddNode   (list_t list, _btree_t newnode, _btree_t listnode)
/*      ~~~~~~~~~~~~
 *  This is a routine that tries to add a node starting at the
 *  current point in the tree.  It works by seeing if it should
 *  be added at this level, and if not descending a level by
 *  calling itself recursively.
 *----------------------------------------------------------------------*/
{
    if (listnode == NULL)
    {
        /*
         *  List empty
         */
        list->nodes = newnode;
    }
    else
    {
        /*
         *  List not empty
         */
        if (LIST_Compare (list, _LIST_USERNODE(list,newnode),
                                _LIST_USERNODE(list,listnode)) < 0)
        {
            if (listnode->left != NULL)
            {
                BTREE_AddNode(list,newnode,listnode->left);
            }
            else
            {
                listnode->left = newnode;
                newnode->parent = listnode;
            }
        }
        else
        {
            if (listnode->right != NULL)
            {
                BTREE_AddNode(list,newnode,listnode->right);
            }
            else
            {
                listnode->right = newnode;
                newnode->parent = listnode;
            }
        }
    }
    return;
}


/*========================================================= BTREE_AFTER */
extern
int     BTREE_After     (list_t list, node_t newnode, node_t oldnode)
/*      ~~~~~~~~~~~
 *  Add a node after the given node.
 *-----------------------------------------------------------------------*/
#define _newnode  ((_btree_t)newnode)
#define _oldnode  ((_btree_t)oldnode)
{
    assert ((list->nodes == NULL && oldnode == NULL)
          ||(list->nodes != NULL && oldnode != NULL) );

    debug ("BTREE_After", \
           "'newnode' appears to already be linked in to a list", \
           _newnode->parent == NULL && _newnode->left == NULL && _newnode->right == NULL);

    BTREE_AddNode (list, _newnode, _oldnode);

    return (0);
}
#undef _newnode
#undef _oldnode


/*=========================================================== BTREE_ADD */
extern
int     BTREE_Add       (list_t list, node_t node)
/*      ~~~~~~~~~
 *  Add a node at the correct point.
 *----------------------------------------------------------------------*/
#define _list ((_btreeh_t)list)
#define _node ((_btree_t)node)
{
    debug ("BTREE_Add", \
           "'node' appears to already be linked in to a list", \
           _node->parent == NULL && _node->left == NULL && _node->right == NULL);

    BTREE_AddNode(list, _node, _list->nodes);

    return (0);
}
#undef _list
#undef _node



/*========================================================================
 *
 *  This is the list of vectored functions
 *  that are used to support this list type.
 *
 *------------------------------------------------------------------------*/

static struct _listmaster master =
        {
        sizeof(struct _btreeh),     /* Size of the list header */
        sizeof(struct _btree),      /* Size of the node header */
        &_LIST_ListCount,
        &_LIST_BtreeFirst,
        &_LIST_BtreeLast,
        &_LIST_BtreeNext,
        &_LIST_BtreePrevious,
        NULL,
        NULL,
        &BTREE_Add,
        &BTREE_After,
        &_LIST_BtreeRemove,
        &_LIST_BtreeFind,
        _LIST_BTREE,
        "BTREE",
        "Binary Tree"
        };

/*
 *  This is the default settings for a new
 *  object of this type.
 */
struct _listclass _LISTbtree =
        {
        &master,                    /* Pointer to master class data */
        0,                          /* Clone count */
        0,                          /* Size of user node data */
        NULL,                       /* User node Initialise */
        NULL,                       /* User node Destroy */
        NULL,                       /* user node Compare */
        NULL                        /* Linked list (none) */
        };


