/*
 *      L I S T _ C l o n e
 *
 *  This function is called when it is required
 *  to clone an existing list.
 *
 *
 * (c) Copyright 1996 David J. Walker
 *     Email:  d.j.walker@slh0101.wins.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  28 May 96   DJW   - First version
 */

#include <liblist.h>
#include <sys/liblist.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

/*
 *  Internal support routine that does the
 *  clone process given the class pointer
 */
list_t  _LIST_Clone  (_listclass_t class)
{
    list_t  list;

    /*
     *  Set up a new list instance, and
     *  ensure that all fields are clear
     */
    list = calloc (class->master->sizelist,1);
    if (list == NULL)
    {
        return NULL;
    }
    /*
     *  Set the header pointer up
     */
    ((_list1h_t)list)->class = class;
    /*
     *  See if anything else to do ?
     */
    if (class->master->clone != NULL)
    {
        int     reply;

        reply = (*class->master->clone)(list);
        if (reply != 0)
        {
            errno = reply;
            free (list);
            return (NULL);
        }
    }
    return list;
}


list_t  LIST_Clone  (list_t oldlist)
{
    debug ("LIST_Clone", \
           "'oldlist' does not point to a valid list type", \
           _LIST_ListType(oldlist));


    return _LIST_Clone(oldlist->class);
}



