/*
 *      L I S T _ C r e a t e
 *
 *  This function is called when it is required
 *  to create a list.
 *
 *
 * (c) Copyright 1996 David J. Walker
 *     Email:  d.j.walker@slh0101.wins.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 */

#include <liblist.h>
#include <sys/liblist.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

list_t  LIST_Create  (struct _listclass * listtype, 
                      size_t nodesize,
                      int (*initfunc)(node_t,va_list), 
                      int (*killfunc)(node_t),
                      int (*compfunc)(node_t, node_t) )
{
    _listclass_t class;
    list_t  list;

    debug ("LIST_Create", \
           "type parameter is NULL", \
           listtype != NULL);

    if (nodesize == 0)
    {
        errno = EINVAL;
        return NULL;
    }
    /*
     *  We MUST have a 'Node Initialisation'
     *  function for hash tables.
     */
    switch (listtype->master->typeid)
    {
    case _LIST_HASH:
            if (initfunc == NULL)
            {
                errno = EINVAL;
                return NULL;
            }
            break;
    default:
            break;
    }
    /*
     *  Handle list types for which we
     *  do not support both ordered and
     *  unordered variants.
     */
    switch (listtype->master->typeid)
    {
    case _LIST_FIFO:
    case _LIST_LIFO:
            if (compfunc != NULL)
            {
                errno = EINVAL;
                return NULL;
            }
            break;

    case _LIST_HASH:
            if (compfunc == NULL)
            {
                errno = EINVAL;
                return NULL;
            }
            break;

    default:
            break;
    }

    /*
     *  Create a new class structure
     */
    class = malloc (sizeof(struct _listclass));
    if (class == NULL)
    {
        return NULL;
    }
    (void) memcpy (class, listtype, sizeof (struct _listclass));

    /*
     *  Initialise it with the
     *  user supplied data.
     */
    class->sizeuser = nodesize;
    class->initnode = initfunc;
    if (killfunc != NULL)
    {
        class->killnode = killfunc;
    }
    if (compfunc != NULL)
    {
        class->compare = compfunc;
    }

    /*
     *  Now set up this instance
     */
    list = _LIST_Clone(class);
    if (list == NULL)
    {
        free (class);
        return NULL;
    }

    return list;
}



