/*
 *      L I S T _ D e s t r o y
 *
 *  This function is called when it is required
 *  to destroy a list.
 *
 *
 * (c) Copyright 1996 David J. Walker
 *     Email:  d.j.walker@slh0101.wins.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 */

#include <liblist.h>
#include <sys/liblist.h>
#include <stdlib.h>
#include <stdio.h>


int  LIST_Destroy  (list_t list)
{
    register node_t  listnode;
    register _listclass_t  class;
    register _listmaster_t master;

    debug ("LIST_Destroy", \
           "list does not point to a valid list instance",\
           _LIST_ListType(list));

    class = list->class;
    master = class->master;
    while ((listnode = (*master->first)(list)) != NULL)
    {
        int     result;
        /*
         *  Remove the node from the list
         *  (this cannot fail ?)
         */
        result = (*master->remove)(list, _LIST_REALNODE(list,listnode));
        assert (result == 0);
        /*
         *  Kill off the node
         */
        result = LIST_FreeNode (list,listnode);
        /*
         *  Check no error as if so we had better
         *  abandon before catastrophes happen.
         */
        if (result != 0)
        {
            return result;
        }
    }
    /*
     *  We have freed all the nodes, so we
     *  can destroy the list object itself
     */
    free (list);
    /*
     *  If this is the last clone,
     *  then free class as well
     */
    class->clones -= 1;
    if (class->clones == 0)
    {
        free (list->class);
    }

    return 0;
}

