/*
 *      L I S T _ E n u m e r a t e
 *
 *  This function is called when it is required
 *  to enumerate a list.
 *
 *  If the list is unordered, then this is simply
 *  a walk through the list in order.
 *
 *  Returns:
 *      -ve     Error occurred
 *      0       OK
 *      +ve     Result from user function
 *
 *
 * (c) Copyright 1996 David J. Walker
 *     Email:  d.j.walker@slh0101.wins.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 */


#include <liblist.h>
#include <sys/liblist.h>


long   LIST_Enumerate (list_t list, int (*enumfunc)(node_t, va_list),...)
{
    register node_t  node;

    debug ("LIST_Enumerate", \
           "'list' parameter is not a valid list instance", \
           _LIST_ListType(list));
    debug ("LIST_Enumerate", \
           "the node enumeration function parameter is NULL", \
           enumfunc !=  NULL);

    for (node = LIST_First(list) ; node != NULL ; node = LIST_Next(list,node))
    {
        va_list ap;
        int     result;

        va_start(ap, enumfunc);
        result = (*enumfunc)(node,ap);
        va_end(ap);
        if (result != 0)
        {
            return result;
        }
    }
    return 0;
}

