/*
 *      _ L I S T _ F i f o R e m o v e
 *
 *  This is used to remove a node from a 
 *  singled linked list that does NOT have
 *  a pointer to the last node (FIFO).
 *
 *
 * (c) Copyright 1996 David J. Walker
 *     Email:  d.j.walker@slh0101.wins.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  23 May 96   DJW   - First version
 *
 *  28 May 96   DJW   - Updated 'remove' to allow for the user providing
 *                      a member to remove who is not on the list!
 */


#include <liblist.h>
#include <sys/liblist.h>
#include <errno.h>


int     _LIST_FifoRemove (list_t list, node_t node)

#define _list  ((_fifoh_t)list)
#define usernode  ((_fifo_t)node)
{
    _fifo_t listnode;

    /*
     *  If this is the first node, then
     *  we have an easy life!
     */
    if (usernode == list->nodes)
    {
        list->nodes = usernode->next;
    }
    else
    {
        /*
         *  We need to scan the whole list in sequence
         */
        for (listnode = list->nodes ; listnode != NULL ; listnode = listnode->next)
        {
            if (listnode->next == usernode)
            {
                break;
            }
        }
        /*
         *  We had better check that we found it!
         *  The user could give us a member not attached
         *  (or a NULL value which has the same effect).
         */
        if (listnode == NULL)
        {
            return (node == NULL ? EINVAL : ENOENT);
        }
        listnode->next = usernode->next;
    }
    /*
     *  Now do the tidying up.
     */
    if (_list->last == usernode)
    {
        assert (usernode->next == NULL);
        _list->last = listnode;
    }
    usernode->next = NULL;
    return (0);
}
#undef _list
#undef usernode

