
#ifndef _SYS_LIBLIST_H
#define _SYS_LIBLIST_H
#ifndef _LIBLIST_H
#include <liblist.h>
#endif
#include <sys/types.h>
#include <stdarg.h>
#include <assert.h>
#ifdef __STDC__
#define _P_(params) params
#else
#define _P_(params)
#endif
#ifndef DEBUG
#define debug(routine,message,expr) ((void) 0)
#else
#undef debug
#define _LISTSTR(x) _LISTVAL(x)
#define _LISTVAL(x) #x
#define debug(routine,message,expr) ((expr) ? (void)0 \
: _assert( __FILE__":" _LISTSTR(__LINE__)"\n" \
routine":" message))
#endif
typedef struct _listclass * _listclass_t;
struct _listmaster
{
size_t sizelist;
size_t sizenode;
size_t (*count) _P_((list_t));
node_t (*first) _P_((list_t));
node_t (*last) _P_((list_t));
node_t (*next) _P_((list_t,node_t));
node_t (*previous) _P_((list_t,node_t));
int (*clone) _P_((list_t));
int (*nodenew) _P_((list_t,node_t,va_list));
int (*add) _P_((list_t,node_t));
int (*after) _P_((list_t,node_t,node_t));
int (*remove) _P_((list_t,node_t));
node_t (*find) _P_((list_t,int (*findfunc)(node_t,va_list),va_list));
short typeid;
char * typename;
char * typedesc;
};
typedef struct _listmaster * _listmaster_t;
#define _LIST_SINGLE 1
#define _LIST_DOUBLE 2
#define _LIST_FIFO 4
#define _LIST_LIFO 8
#define _LIST_BTREE 16
#define _LIST_BALTREE 32
#define _LIST_HASH 64
struct _listclass
{
struct _listmaster *master;
size_t sizeuser;
int clones;
int (*initnode)_P_((node_t,va_list));
int (*killnode)_P_((node_t));
int (*compare)_P_((node_t,node_t));
_listclass_t linked;
};
struct _listhead
{
_listclass_t class;
node_t nodes;
};
#ifndef DEBUG
#define _LIST_NODECHECK
#else
typedef struct _nodecheck {
list_t list;
_listclass_t type;
} * _nodecheck_t;
#define _LIST_NODECHECK struct _nodecheck check;
#endif
typedef struct _list1
{
_LIST_NODECHECK
struct _list1 * next;
} * _list1_t;
typedef struct _list1h
{
_listclass_t class;
_list1_t nodes;
} * _list1h_t;
typedef struct _list2
{
_LIST_NODECHECK
struct _list2 * next;
struct _list2 * prev;
} * _list2_t;
typedef struct _list2h
{
_listclass_t class;
_list2_t nodes;
_list2_t last;
} * _list2h_t;
typedef struct _fifo
{
_LIST_NODECHECK
struct _fifo * next;
} * _fifo_t;
typedef struct _fifoh
{
_listclass_t class;
_fifo_t nodes;
_fifo_t last;
} * _fifoh_t;
typedef struct _lifo
{
_LIST_NODECHECK
struct _lifo * next;
} * _lifo_t;
typedef struct _lifoh
{
_listclass_t class;
_lifo_t nodes;
} * _lifoh_t;
typedef struct _btree
{
_LIST_NODECHECK
struct _btree * parent;
struct _btree * left;
struct _btree * right;
} * _btree_t;
typedef struct _btreeh
{
_listclass_t class;
_btree_t nodes;
} * _btreeh_t;
typedef struct _hash
{
_LIST_NODECHECK
list_t linked;
} * _hash_t;
typedef struct _hashh
{
_listclass_t class;
_hash_t nodes;
size_t size;
} * _hashh_t;
#define _LIST_USERNODE(l,n) ((node_t)((char *)n + l->class->master->sizenode))
#define _LIST_REALNODE(l,n) ((node_t)((char *)n - l->class->master->sizenode))
list_t _LIST_Clone _P_((_listclass_t));
node_t _LIST_NewNode _P_((list_t,va_list));
size_t _LIST_ListCount _P_((list_t));
int _LIST_Index _P_((list_t,node_t));
node_t _LIST_FifoLast _P_((list_t));
int _LIST_FifoAfter _P_((list_t,node_t,node_t));
int _LIST_FifoRemove _P_((list_t,node_t));
node_t _LIST_LifoLast _P_((list_t));
node_t _LIST_LifoPrevious _P_((list_t,node_t));
int _LIST_LifoAfter _P_((list_t,node_t,node_t));
int _LIST_LifoRemove _P_((list_t,node_t));
node_t _LIST_ListFirst _P_((list_t));
node_t _LIST_ListNext _P_((list_t,node_t));
int _LIST_ListAdd _P_((list_t,node_t));
node_t _LIST_ListFind _P_((list_t,int (*)(node_t,va_list),va_list));
node_t _LIST_BtreeFirst _P_((list_t));
node_t _LIST_BtreeLast _P_((list_t));
node_t _LIST_BtreeNext _P_((list_t,node_t));
node_t _LIST_BtreePrevious _P_((list_t,node_t));
int _LIST_BtreeRemove _P_((list_t,node_t));
node_t _LIST_BtreeFind _P_((list_t,int (*)(node_t,va_list),va_list));
node_t _LIST_HashFirst _P_((list_t));
node_t _LIST_HashLast _P_((list_t));
node_t _LIST_HashPrevious _P_((list_t,node_t));
node_t _LIST_HashNext _P_((list_t,node_t));
int _LIST_HashAfter _P_((list_t,node_t,node_t));
int _LIST_HashAdd _P_((list_t,node_t));
int _LIST_HashRemove _P_((list_t,node_t));
node_t _LIST_HashFind _P_((list_t,int (*)(node_t,va_list),va_list));
int _LIST_ListType _P_((list_t));
int _LIST_ListId _P_((list_t,node_t));
#endif
