/*
 *          _ L I S T _ L i f o A f t e r
 *
 *  This routine adds a node on a list that
 *  DOES NOT maintain a pointer to the last node
 *
 *
 * (c) Copyright 1996 David J. Walker
 *     Email:  d.j.walker@slh0101.wins.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  23 May 96   DJW   - First version
 */


#include <liblist.h>
#include <sys/liblist.h>
#include <errno.h>




int     _LIST_LifoAfter (list_t list, node_t newnode, node_t oldnode)

#define _list  ((_lifoh_t)list)
#define _newnode ((_lifo_t)newnode)
#define _oldnode ((_lifo_t)oldnode)
{
    assert (newnode != NULL);
    assert (_newnode->next == NULL);

    /*
     * Empty list ?
     */
    if (list->nodes == NULL)
    {
        assert (oldnode == NULL);

        list->nodes = newnode;
    }
    else
    {
        if (oldnode == NULL)
        {
            /*
             *  At start of existing list
             */
            _newnode->next = _list->nodes;
            _list->nodes = _newnode;
        }
        else
        {
            /*
             *  Into middle of existing list
             */

            _newnode->next = _oldnode->next;
            _oldnode->next = _newnode;
        }
    }
    return (0);
}
#undef _list
#undef _newnode
#undef _oldnode

