/*
 *      L I S T _ N e w A f t e r
 *
 *  This function is called when it is required
 *  to create a new node and immediately insert it
 *  after a specifed node.
 *
 *
 * (c) Copyright 1996 David J. Walker
 *     Email:  d.j.walker@slh0101.wins.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  22 May 96   DJW   - First version
 */


#include <liblist.h>
#include <sys/liblist.h>
#include <errno.h>


/*
 *  This function combines the LIST_New() functionality
 *  with that of the LIST_After() method in one call.
 */
node_t    LIST_NewAfter (list_t list, node_t oldnode, ...)
{
    node_t  newnode;
    int     reply;
    va_list ap;

    debug ("LIST_NewAfter", \
           "'list' parameter is not a valid list instance", \
           _LIST_ListType(list));

    if (oldnode == NULL)
    {
        oldnode = LIST_Last(list);
    }
#ifdef DEBUG
    if (oldnode != NULL)
    {
        debug ("LIST_NewAfter", \
               "'oldnode' is not a valid member of this list", \
               _LIST_ListId(list,oldnode));
        debug  ("LIST_NewAfter", \
               "'oldnode' does not exist in this list", \
               LIST_Index(list,oldnode));
    }
#endif /* DEBUG */


    /*
     *  Check that a node initialisation
     *  routine has been provided.
     */
    if (list->class->initnode == NULL)
    {
        errno = EACCES;
        return NULL;
    }
    /*
     *  Create a new node
     *  (passing on variable parameter list)
     */
    va_start (ap, oldnode);
    newnode = _LIST_NewNode(list,ap);
    va_end(ap);
    if (newnode == NULL)
    {
        return (NULL);
    }
    reply = LIST_Before(list,newnode,oldnode);
    if (reply != 0)
    {
        errno = reply;
        (void)LIST_FreeNode(list,newnode);
        return (NULL);
    }
    return (newnode);
}


