/*
 *      L I S T _ N e w N o d e
 *
 *  This function is called when it is required
 *  to create a new nodelist.
 *
 *
 * (c) Copyright 1996 David J. Walker
 *     Email:  d.j.walker@slh0101.wins.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 */

#include <liblist.h>
#include <sys/liblist.h>
#include <stdlib.h>

/*
 *  Internal support version of the new
 *  node creation routine which has the
 *  variable parameter list dereferenced.
 */
extern
node_t  _LIST_NewNode (list_t list, va_list ap)
{
    register node_t  node;
    register node_t  nodeuser;
    register _listclass_t class;
    register _listmaster_t master;

    class = list->class;
    master = class->master;

    node = calloc (1,master->sizenode + class->sizeuser);
    if (node == NULL)
    {
        return NULL;
    }
#ifdef DEBUG
    /*
     *  Set the node ID check.
     *  N.B.  This assumes that ALL node types
     *      have the ID field as the first field.
     */
    ((_nodecheck_t)node)->list = list;
    ((_nodecheck_t)node)->type = class;
#endif /* DEBUG */

    nodeuser = _LIST_USERNODE(list,node);
    /*
     *  Check for a user supplied
     *  initialisation routine.
     */
    if (class->initnode)
    {
        int     result;

        result = (*class->initnode)(nodeuser,ap);
        /*
         *  If user routine returns error
         *  then we must tidy and propgate
         *  the error back to the user.
         */
        if (result != 0)
        {
            free (node);
            return (NULL);
        }
    }

    return (nodeuser);
}

/*
 *  Public version of new node routine
 *  This simply dereferences the parameter
 *  list and then calls the internal routine
 */
extern 
node_t  LIST_NewNode (list_t list, ...)
{
    node_t  node;
    va_list ap;

    debug ("LIST_NewNode", \
           "'list' parameter is not a valid list instance", \
           _LIST_ListType(list));

    va_start(ap,list);
    node = _LIST_NewNode(list, ap);
    va_end(ap);
    return (node);
}

