/*
 *      L I S T _ P o s i t i o n
 *
 *  This function is called when it is required
 *  to find out the position of a node in the list.
 *
 *  Returns:
 *      0       No
 *      <>0     Yes
 *
 *
 * (c) Copyright 1996 David J. Walker
 *     Email:  d.j.walker@slh0101.wins.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 */


#include <liblist.h>
#include <sys/liblist.h>
#include <errno.h>

node_t    LIST_Position (list_t list, size_t index)
{
    node_t  listnode;

    debug ("LIST_Position", \
           "'list' parameter is not a valid list instance", \
           _LIST_ListType(list));
    debug ("LIST_Position", \
           "'index' parameter is 0", \
           index != 0);
    debug ("LIST_Position", \
           "'index' parameter is too large", \
           index <= LIST_Index(list,LIST_Last(list)));

    for (listnode = LIST_First(list) ; listnode != NULL ; 
                        listnode = LIST_Next(list,listnode))
    {
        if (--index <= 0)
        {
            return (listnode);
        }
    }
    /*
     *  The node was not found if
     *  this point is reached.
     */
    errno = EINVAL;
    return NULL;
}

